"""
Sage Intacct Glentry
"""
from typing import Dict

from .api_base import ApiBase

class GlentryRoyaltiesCN(ApiBase):
    """Class for Contacts APIs."""
    def __init__(self):
        ApiBase.__init__(self, dimension='GLENTRY')

    def get_all(self):
        """Get all GLENTRY ROYALTIES CREDIT NOTES records from Sage Intacct

        Returns:
            List of Dict in GLENTRY.
        """

        complete_data = []

        pagesize = '1000'
        data = {
            'readByQuery': {
                'object': 'GLENTRY',
                'fields': '*',
                'query': 'ACCOUNTNO = 8000 AND ITEMID = 1000',
                'pagesize': pagesize,
            }
        }
        firstResult = self.format_and_send_request(data)
        complete_data.extend(firstResult['data']['glentry'])

        numRemaining = firstResult['data']['@numremaining']
        resultId = firstResult['data']['@resultId']
        while int(numRemaining) > 0:
            data = {
                'readMore': {
                    'resultId': resultId
                }
            }
            nextResult = self.format_and_send_request(data)
            complete_data.extend(nextResult['data']['glentry'])
            numRemaining = nextResult['data']['@numremaining']
            resultId = nextResult['data']['@resultId']

        return complete_data
