"""
Sage Intacct Python SDK
"""
from .apis import ApiBase, Contacts, Locations, Employees, Accounts, ExpenseTypes, Attachments, ExpenseReports,\
    Vendors, Bills, Projects, Departments, ChargeCardAccounts, ChargeCardTransactions, Customers, Items,\
    APPayments, Reimbursements, CheckingAccounts, SavingsAccounts, Tasks, ExpensePaymentTypes, Dimensions,\
    DimensionValues, LocationEntities, ARInvoices, ARPayments, TaxDetails, GLDetail, Classes, JournalEntries,\
    RevRecSchedules, RevRecScheduleEntries, CostTypes, OrderEntryTransactions, Warehouse, Sodocument,\
    Sodocumententry, Podocument, Podocumententry, Invdocument, Invdocumententry, Itemwarehouseinfo, Glresolve,\
    Allocation, Allocationentry, Glacctgrp, Glacctgrphierarchy, Glbatch, Glbudgetheader, Glbudgetitem,\
    Glentry, GlentryRoyalties, GlentryMarketing, Gljournal, Reportingperiod, Stataccount, Apdetail, Aprecord,\
    Ardetail, Arrecord, Cmdetail, Cmrecord,Eedetail, Eerecord, SodocumententryRoyalties, SodocumententryMarketing,\
    Invdocumentsubtotals, Podocumentsubtotals, Sodocumentsubtotals, Glaccount, GLDetailMarketing, GLDetailRoyalties, \
    GlbudgetitemMarketing, GlbudgetitemRoyalties, GlbudgetitemTurnover


class SageIntacctSDK:
    """
    Sage Intacct SDK
    """

    def __init__(self, sender_id: str, sender_password: str, user_id: str,
        company_id: str, user_password: str, entity_id: str=None, show_private: bool=False):
        """
        Initialize connection to Sage Intacct
        :param sender_id: Sage Intacct sender id
        :param sender_password: Sage Intacct sener password
        :param user_id: Sage Intacct user id
        :param company_id: Sage Intacct company id
        :param user_password: Sage Intacct user password
        :param (optional) entity_id: Sage Intacct entity ID
        :param (optional) show_private: Show Private on API Calls
        """
        # Initializing variables
        self.__sender_id = sender_id
        self.__sender_password = sender_password
        self.__user_id = user_id
        self.__company_id = company_id
        self.__user_password = user_password
        self.__entity_id = entity_id
        self.__show_private = show_private

        self.api_base = ApiBase()
        self.apdetail = Apdetail()
        self.aprecord = Aprecord()
        self.ardetail = Ardetail()
        self.arrecord = Arrecord()
        self.cmdetail = Cmdetail()
        self.cmrecord = Cmrecord()
        self.eedetail = Eedetail()
        self.eerecord = Eerecord()
        self.warehouse = Warehouse()
        self.allocation = Allocation()
        self.allocationentry = Allocationentry()
        self.glacctgrp = Glacctgrp()
        self.glacctgrphierarchy = Glacctgrphierarchy()
        self.glaccount = Glaccount()
        self.glbatch = Glbatch()
        self.glbudgetheader = Glbudgetheader()
        self.glbudgetitem = Glbudgetitem()
        self.glbudgetitemMarketing = GlbudgetitemMarketing()
        self.glbudgetitemRoyalties = GlbudgetitemRoyalties()
        self.glbudgetitemTurnover= GlbudgetitemTurnover()
        self.glentry = Glentry()
        self.glentryRoyalties = GlentryRoyalties()
        self.glentryMarketing = GlentryMarketing()
        self.gljournal = Gljournal()
        self.glresolve = Glresolve()
        self.reportingperiod = Reportingperiod()
        self.stataccount = Stataccount()
        self.sodocumententry = Sodocumententry()
        self.sodocumententryRoyalties = SodocumententryRoyalties()
        self.sodocumententryMarketing = SodocumententryMarketing()
        self.sodocument = Sodocument()
        self.sodocumentsubtotals = Sodocumentsubtotals()
        self.podocumentsubtotals = Podocumentsubtotals()
        self.invdocumentsubtotals = Invdocumentsubtotals()
        self.podocumententry = Podocumententry()
        self.podocument = Podocument()
        self.invdocumententry = Invdocumententry()
        self.invdocument = Invdocument()
        self.itemwarehouseinfo = Itemwarehouseinfo()
        self.contacts = Contacts()
        self.locations = Locations()
        self.employees = Employees()
        self.accounts = Accounts()
        self.expense_types = ExpenseTypes()
        self.attachments = Attachments()
        self.expense_reports = ExpenseReports()
        self.vendors = Vendors()
        self.bills = Bills()
        self.projects = Projects()
        self.departments = Departments()
        self.charge_card_accounts = ChargeCardAccounts()
        self.charge_card_transactions = ChargeCardTransactions()
        self.customers = Customers()
        self.items = Items()
        self.ap_payments = APPayments()
        self.ar_invoices = ARInvoices()
        self.ar_payments = ARPayments()
        self.reimbursements = Reimbursements()
        self.checking_accounts = CheckingAccounts()
        self.savings_accounts = SavingsAccounts()
        self.dimensions = Dimensions()
        self.dimension_values = DimensionValues()
        self.tasks = Tasks()
        self.expense_payment_types = ExpensePaymentTypes()
        self.location_entities = LocationEntities()
        self.tax_details = TaxDetails()
        self.gl_detail = GLDetail()
        self.gl_detailMarketing = GLDetailMarketing()
        self.gl_detailRoyalties = GLDetailRoyalties()
        self.classes = Classes()
        self.journal_entries = JournalEntries()
        self.rev_rec_schedules = RevRecSchedules()
        self.rev_rec_schedule_entries = RevRecScheduleEntries()
        self.cost_types = CostTypes()
        self.order_entry_transactions = OrderEntryTransactions()
        self.update_sender_id()
        self.update_sender_password()
        self.update_session_id()
        self.update_show_private()

    def update_sender_id(self):
        """
        Update the sender id in all API objects.
        """
        self.api_base.set_sender_id(self.__sender_id)
        self.apdetail.set_sender_id(self.__sender_id)
        self.aprecord.set_sender_id(self.__sender_id)
        self.ardetail.set_sender_id(self.__sender_id)
        self.arrecord.set_sender_id(self.__sender_id)
        self.cmdetail.set_sender_id(self.__sender_id)
        self.cmrecord.set_sender_id(self.__sender_id)
        self.eedetail.set_sender_id(self.__sender_id)
        self.eerecord.set_sender_id(self.__sender_id)
        self.warehouse.set_sender_id(self.__sender_id)
        self.allocation.set_sender_id(self.__sender_id)
        self.allocationentry.set_sender_id(self.__sender_id)
        self.glacctgrp.set_sender_id(self.__sender_id)
        self.glacctgrphierarchy.set_sender_id(self.__sender_id)
        self.glaccount.set_sender_id(self.__sender_id)
        self.glbatch.set_sender_id(self.__sender_id)
        self.glbudgetheader.set_sender_id(self.__sender_id)
        self.glbudgetitem.set_sender_id(self.__sender_id)
        self.glbudgetitemMarketing.set_sender_id(self.__sender_id)
        self.glbudgetitemRoyalties.set_sender_id(self.__sender_id)
        self.glbudgetitemTurnover.set_sender_id(self.__sender_id)
        self.glentry.set_sender_id(self.__sender_id)
        self.glentryRoyalties.set_sender_id(self.__sender_id)
        self.glentryMarketing.set_sender_id(self.__sender_id)
        self.gljournal.set_sender_id(self.__sender_id)
        self.glresolve.set_sender_id(self.__sender_id)
        self.reportingperiod.set_sender_id(self.__sender_id)
        self.stataccount.set_sender_id(self.__sender_id)
        self.sodocumententry.set_sender_id(self.__sender_id)
        self.sodocumententryRoyalties.set_sender_id(self.__sender_id)
        self.sodocumententryMarketing.set_sender_id(self.__sender_id)
        self.sodocument.set_sender_id(self.__sender_id)
        self.sodocumentsubtotals.set_sender_id(self.__sender_id)
        self.podocumentsubtotals.set_sender_id(self.__sender_id)
        self.invdocumentsubtotals.set_sender_id(self.__sender_id)
        self.podocumententry.set_sender_id(self.__sender_id)
        self.podocument.set_sender_id(self.__sender_id)
        self.invdocumententry.set_sender_id(self.__sender_id)
        self.invdocument.set_sender_id(self.__sender_id)
        self.itemwarehouseinfo.set_sender_id(self.__sender_id)
        self.contacts.set_sender_id(self.__sender_id)
        self.locations.set_sender_id(self.__sender_id)
        self.employees.set_sender_id(self.__sender_id)
        self.accounts.set_sender_id(self.__sender_id)
        self.expense_types.set_sender_id(self.__sender_id)
        self.attachments.set_sender_id(self.__sender_id)
        self.expense_reports.set_sender_id(self.__sender_id)
        self.vendors.set_sender_id(self.__sender_id)
        self.bills.set_sender_id(self.__sender_id)
        self.projects.set_sender_id(self.__sender_id)
        self.departments.set_sender_id(self.__sender_id)
        self.charge_card_accounts.set_sender_id(self.__sender_id)
        self.charge_card_transactions.set_sender_id(self.__sender_id)
        self.customers.set_sender_id(self.__sender_id)
        self.items.set_sender_id(self.__sender_id)
        self.ap_payments.set_sender_id(self.__sender_id)
        self.ar_invoices.set_sender_id(self.__sender_id)
        self.ar_payments.set_sender_id(self.__sender_id)
        self.reimbursements.set_sender_id(self.__sender_id)
        self.checking_accounts.set_sender_id(self.__sender_id)
        self.savings_accounts.set_sender_id(self.__sender_id)
        self.dimensions.set_sender_id(self.__sender_id)
        self.dimension_values.set_sender_id(self.__sender_id)
        self.tasks.set_sender_id(self.__sender_id)
        self.expense_payment_types.set_sender_id(self.__sender_id)
        self.location_entities.set_sender_id(self.__sender_id)
        self.tax_details.set_sender_id(self.__sender_id)
        self.gl_detail.set_sender_id(self.__sender_id)
        self.gl_detailMarketing.set_sender_id(self.__sender_id)
        self.gl_detailRoyalties.set_sender_id(self.__sender_id)
        self.classes.set_sender_id(self.__sender_id)
        self.journal_entries.set_sender_id(self.__sender_id)
        self.rev_rec_schedules.set_sender_id(self.__sender_id)
        self.rev_rec_schedule_entries.set_sender_id(self.__sender_id)
        self.cost_types.set_sender_id(self.__sender_id)
        self.order_entry_transactions.set_sender_id(self.__sender_id)

    def update_sender_password(self):
        """
        Update the sender password in all API objects.
        """
        self.api_base.set_sender_password(self.__sender_password)
        self.apdetail.set_sender_password(self.__sender_password)
        self.aprecord.set_sender_password(self.__sender_password)
        self.ardetail.set_sender_password(self.__sender_password)
        self.arrecord.set_sender_password(self.__sender_password)
        self.cmdetail.set_sender_password(self.__sender_password)
        self.cmrecord.set_sender_password(self.__sender_password)
        self.eedetail.set_sender_password(self.__sender_password)
        self.eerecord.set_sender_password(self.__sender_password)
        self.warehouse.set_sender_password(self.__sender_password)
        self.allocation.set_sender_password(self.__sender_password)
        self.allocationentry.set_sender_password(self.__sender_password)
        self.glacctgrp.set_sender_password(self.__sender_password)
        self.glacctgrphierarchy.set_sender_password(self.__sender_password)
        self.glaccount.set_sender_password(self.__sender_password)
        self.glbatch.set_sender_password(self.__sender_password)
        self.glbudgetheader.set_sender_password(self.__sender_password)
        self.glbudgetitem.set_sender_password(self.__sender_password)
        self.glbudgetitemMarketing.set_sender_password(self.__sender_password)
        self.glbudgetitemRoyalties.set_sender_password(self.__sender_password)
        self.glbudgetitemTurnover.set_sender_password(self.__sender_password)
        self.glentry.set_sender_password(self.__sender_password)
        self.glentryRoyalties.set_sender_password(self.__sender_password)
        self.glentryMarketing.set_sender_password(self.__sender_password)
        self.gljournal.set_sender_password(self.__sender_password)
        self.glresolve.set_sender_password(self.__sender_password)
        self.reportingperiod.set_sender_password(self.__sender_password)
        self.stataccount.set_sender_password(self.__sender_password)
        self.sodocumententry.set_sender_password(self.__sender_password)
        self.sodocumententryRoyalties.set_sender_password(self.__sender_password)
        self.sodocumententryMarketing.set_sender_password(self.__sender_password)
        self.sodocument.set_sender_password(self.__sender_password)
        self.podocumententry.set_sender_password(self.__sender_password)
        self.podocument.set_sender_password(self.__sender_password)
        self.podocumentsubtotals.set_sender_password(self.__sender_password)
        self.sodocumentsubtotals.set_sender_password(self.__sender_password)
        self.invdocumentsubtotals.set_sender_password(self.__sender_password)
        self.invdocumententry.set_sender_password(self.__sender_password)
        self.invdocument.set_sender_password(self.__sender_password)
        self.itemwarehouseinfo.set_sender_password(self.__sender_password)
        self.contacts.set_sender_password(self.__sender_password)
        self.locations.set_sender_password(self.__sender_password)
        self.employees.set_sender_password(self.__sender_password)
        self.accounts.set_sender_password(self.__sender_password)
        self.expense_types.set_sender_password(self.__sender_password)
        self.attachments.set_sender_password(self.__sender_password)
        self.expense_reports.set_sender_password(self.__sender_password)
        self.vendors.set_sender_password(self.__sender_password)
        self.bills.set_sender_password(self.__sender_password)
        self.projects.set_sender_password(self.__sender_password)
        self.departments.set_sender_password(self.__sender_password)
        self.charge_card_accounts.set_sender_password(self.__sender_password)
        self.charge_card_transactions.set_sender_password(self.__sender_password)
        self.customers.set_sender_password(self.__sender_password)
        self.items.set_sender_password(self.__sender_password)
        self.ap_payments.set_sender_password(self.__sender_password)
        self.ar_invoices.set_sender_password(self.__sender_password)
        self.ar_payments.set_sender_password(self.__sender_password)
        self.reimbursements.set_sender_password(self.__sender_password)
        self.checking_accounts.set_sender_password(self.__sender_password)
        self.savings_accounts.set_sender_password(self.__sender_password)
        self.dimensions.set_sender_password(self.__sender_password)
        self.dimension_values.set_sender_password(self.__sender_password)
        self.tasks.set_sender_password(self.__sender_password)
        self.expense_payment_types.set_sender_password(self.__sender_password)
        self.location_entities.set_sender_password(self.__sender_password)
        self.tax_details.set_sender_password(self.__sender_password)
        self.gl_detail.set_sender_password(self.__sender_password)
        self.gl_detailMarketing.set_sender_password(self.__sender_password)
        self.gl_detailRoyalties.set_sender_password(self.__sender_password)
        self.classes.set_sender_password(self.__sender_password)
        self.journal_entries.set_sender_password(self.__sender_password)
        self.rev_rec_schedules.set_sender_password(self.__sender_password)
        self.rev_rec_schedule_entries.set_sender_password(self.__sender_password)
        self.cost_types.set_sender_password(self.__sender_password)
        self.order_entry_transactions.set_sender_password(self.__sender_password)

    def update_session_id(self):
        """
        Update the session id and change it in all API objects.
        """
        self.__session_id = self.api_base.get_session_id(
        self.__user_id, self.__company_id, self.__user_password, self.__entity_id)
        self.api_base.set_session_id(self.__session_id)
        self.apdetail.set_session_id(self.__session_id)
        self.aprecord.set_session_id(self.__session_id)
        self.ardetail.set_session_id(self.__session_id)
        self.arrecord.set_session_id(self.__session_id)
        self.cmdetail.set_session_id(self.__session_id)
        self.cmrecord.set_session_id(self.__session_id)
        self.eedetail.set_session_id(self.__session_id)
        self.eerecord.set_session_id(self.__session_id)
        self.warehouse.set_session_id(self.__session_id)
        self.allocation.set_session_id(self.__session_id)
        self.allocationentry.set_session_id(self.__session_id)
        self.glacctgrp.set_session_id(self.__session_id)
        self.glacctgrphierarchy.set_session_id(self.__session_id)
        self.glaccount.set_session_id(self.__session_id)
        self.glbatch.set_session_id(self.__session_id)
        self.glbudgetheader.set_session_id(self.__session_id)
        self.glbudgetitem.set_session_id(self.__session_id)
        self.glbudgetitemMarketing.set_session_id(self.__session_id)
        self.glbudgetitemRoyalties.set_session_id(self.__session_id)
        self.glbudgetitemTurnover.set_session_id(self.__session_id)
        self.glentry.set_session_id(self.__session_id)
        self.glentryRoyalties.set_session_id(self.__session_id)
        self.glentryMarketing.set_session_id(self.__session_id)
        self.gljournal.set_session_id(self.__session_id)
        self.glresolve.set_session_id(self.__session_id)
        self.reportingperiod.set_session_id(self.__session_id)
        self.stataccount.set_session_id(self.__session_id)
        self.sodocumententry.set_session_id(self.__session_id)
        self.sodocumententryRoyalties.set_session_id(self.__session_id)
        self.sodocumententryMarketing.set_session_id(self.__session_id)
        self.sodocument.set_session_id(self.__session_id)
        self.sodocumentsubtotals.set_session_id(self.__session_id)
        self.podocumentsubtotals.set_session_id(self.__session_id)
        self.invdocumentsubtotals.set_session_id(self.__session_id)
        self.podocumententry.set_session_id(self.__session_id)
        self.podocument.set_session_id(self.__session_id)
        self.invdocumententry.set_session_id(self.__session_id)
        self.invdocument.set_session_id(self.__session_id)
        self.itemwarehouseinfo.set_session_id(self.__session_id)
        self.contacts.set_session_id(self.__session_id)
        self.locations.set_session_id(self.__session_id)
        self.employees.set_session_id(self.__session_id)
        self.accounts.set_session_id(self.__session_id)
        self.expense_types.set_session_id(self.__session_id)
        self.attachments.set_session_id(self.__session_id)
        self.expense_reports.set_session_id(self.__session_id)
        self.vendors.set_session_id(self.__session_id)
        self.bills.set_session_id(self.__session_id)
        self.projects.set_session_id(self.__session_id)
        self.departments.set_session_id(self.__session_id)
        self.charge_card_accounts.set_session_id(self.__session_id)
        self.charge_card_transactions.set_session_id(self.__session_id)
        self.customers.set_session_id(self.__session_id)
        self.items.set_session_id(self.__session_id)
        self.ap_payments.set_session_id(self.__session_id)
        self.ar_invoices.set_session_id(self.__session_id)
        self.ar_payments.set_session_id(self.__session_id)
        self.reimbursements.set_session_id(self.__session_id)
        self.checking_accounts.set_session_id(self.__session_id)
        self.savings_accounts.set_session_id(self.__session_id)
        self.dimensions.set_session_id(self.__session_id)
        self.dimension_values.set_session_id(self.__session_id)
        self.tasks.set_session_id(self.__session_id)
        self.expense_payment_types.set_session_id(self.__session_id)
        self.location_entities.set_session_id(self.__session_id)
        self.tax_details.set_session_id(self.__session_id)
        self.gl_detail.set_session_id(self.__session_id)
        self.gl_detailMarketing.set_session_id(self.__session_id)
        self.gl_detailRoyalties.set_session_id(self.__session_id)
        self.classes.set_session_id(self.__session_id)
        self.journal_entries.set_session_id(self.__session_id)
        self.rev_rec_schedules.set_session_id(self.__session_id)
        self.rev_rec_schedule_entries.set_session_id(self.__session_id)
        self.cost_types.set_session_id(self.__session_id)
        self.order_entry_transactions.set_session_id(self.__session_id)

    def update_show_private(self):
        """
        Update the show private in all API objects.
        """
        self.api_base.set_show_private(self.__show_private)
        self.apdetail.set_show_private(self.__show_private)
        self.aprecord.set_show_private(self.__show_private)
        self.ardetail.set_show_private(self.__show_private)
        self.arrecord.set_show_private(self.__show_private)
        self.cmdetail.set_show_private(self.__show_private)
        self.cmrecord.set_show_private(self.__show_private)
        self.eedetail.set_show_private(self.__show_private)
        self.eerecord.set_show_private(self.__show_private)
        self.warehouse.set_show_private(self.__show_private)
        self.allocation.set_show_private(self.__show_private)
        self.allocationentry.set_show_private(self.__show_private)
        self.glacctgrp.set_show_private(self.__show_private)
        self.glacctgrphierarchy.set_show_private(self.__show_private)
        self.glaccount.set_show_private(self.__show_private)
        self.glbatch.set_show_private(self.__show_private)
        self.glbudgetheader.set_show_private(self.__show_private)
        self.glbudgetitem.set_show_private(self.__show_private)
        self.glbudgetitemMarketing.set_show_private(self.__show_private)
        self.glbudgetitemRoyalties.set_show_private(self.__show_private)
        self.glbudgetitemTurnover.set_show_private(self.__show_private)
        self.glentry.set_show_private(self.__show_private)
        self.glentryRoyalties.set_show_private(self.__show_private)
        self.glentryMarketing.set_show_private(self.__show_private)
        self.gljournal.set_show_private(self.__show_private)
        self.glresolve.set_show_private(self.__show_private)
        self.reportingperiod.set_show_private(self.__show_private)
        self.stataccount.set_show_private(self.__show_private)
        self.sodocumententry.set_show_private(self.__show_private)
        self.sodocumententryRoyalties.set_show_private(self.__show_private)
        self.sodocumententryMarketing.set_show_private(self.__show_private)
        self.sodocument.set_show_private(self.__show_private)
        self.sodocumentsubtotals.set_show_private(self.__show_private)
        self.podocumentsubtotals.set_show_private(self.__show_private)
        self.invdocumentsubtotals.set_show_private(self.__show_private)
        self.podocumententry.set_show_private(self.__show_private)
        self.podocument.set_show_private(self.__show_private)
        self.invdocumententry.set_show_private(self.__show_private)
        self.invdocument.set_show_private(self.__show_private)
        self.itemwarehouseinfo.set_show_private(self.__show_private)
        self.contacts.set_show_private(self.__show_private)
        self.locations.set_show_private(self.__show_private)
        self.employees.set_show_private(self.__show_private)
        self.accounts.set_show_private(self.__show_private)
        self.expense_types.set_show_private(self.__show_private)
        self.attachments.set_show_private(self.__show_private)
        self.expense_reports.set_show_private(self.__show_private)
        self.vendors.set_show_private(self.__show_private)
        self.bills.set_show_private(self.__show_private)
        self.projects.set_show_private(self.__show_private)
        self.departments.set_show_private(self.__show_private)
        self.charge_card_accounts.set_show_private(self.__show_private)
        self.charge_card_transactions.set_show_private(self.__show_private)
        self.customers.set_show_private(self.__show_private)
        self.items.set_show_private(self.__show_private)
        self.ap_payments.set_show_private(self.__show_private)
        self.ar_invoices.set_show_private(self.__show_private)
        self.ar_payments.set_show_private(self.__show_private)
        self.reimbursements.set_show_private(self.__show_private)
        self.checking_accounts.set_show_private(self.__show_private)
        self.savings_accounts.set_show_private(self.__show_private)
        self.dimensions.set_show_private(self.__show_private)
        self.dimension_values.set_show_private(self.__show_private)
        self.tasks.set_show_private(self.__show_private)
        self.expense_payment_types.set_show_private(self.__show_private)
        self.location_entities.set_show_private(self.__show_private)
        self.tax_details.set_show_private(self.__show_private)
        self.gl_detail.set_show_private(self.__show_private)
        self.gl_detailMarketing.set_show_private(self.__show_private)
        self.gl_detailRoyalties.set_show_private(self.__show_private)
        self.classes.set_show_private(self.__show_private)
        self.journal_entries.set_show_private(self.__show_private)
        self.rev_rec_schedules.set_show_private(self.__show_private)
        self.rev_rec_schedule_entries.set_show_private(self.__show_private)
        self.cost_types.set_show_private(self.__show_private)
        self.order_entry_transactions.set_show_private(self.__show_private)